# -*- coding: utf-8 -*-

__module_name__ = "Xchat-FM"
__module_version__ = "0.42"
__module_description__ = "Get currently listened music on last.fm"
__module_author__ = "tusbar <tusbartech [at] online [dot] fr>"

import xchat, urllib, re, cgi, htmlentitydefs
from threading import Thread

class get_previous_song(Thread):

    def __init__(self, user):
        Thread.__init__(self)
        self.f = None
        self.asb = "http://ws.audioscrobbler.com/1.0/user/"
        self.rchart = "/recenttracks.txt"
        self.user = ""
        self.user = user
        self.artist = None
        self.title = None
        self.pattern = re.compile("&(\w+?);")
        return

    def die(self, string):
        print string
        if self.f is not None:
            self.f.close()
        return

    def run(self):
        try:
            self.f = urllib.urlopen(self.asb + self.user + self.rchart)
        except:
            return self.die("Could not open %s" % (self.asb + self.user + self.rchart))
        if self.f is not None:
            try:
                raw = self.f.read().split("\n")[0].split(",")
                raw_song = raw[1].split(u"\u2013".encode("utf-8"))
                self.artist = raw_song[0].rstrip()
                self.title = raw_song[1].lstrip()
                if self.artist == None or self.artist == None:
                    return self.die("Aborting.")
            except: return self.die("Could not retrieve infos")
            self.f.close()
        self.print_infos()
        return

    def descape_entity(self, m, defs=htmlentitydefs.entitydefs):
        try:
            return defs[m.group(1)]
        except KeyError:
            return m.group(0)
        
    def descape(self, string):
        return self.pattern.sub(self.descape_entity, string)    
    
    def print_infos(self):
        import xchat
        xchat.command("me listened to: %s %s %s" % (self.descape(self.artist), u"\u2013".encode("utf-8"), self.descape(self.title)))
        return


class get_current_song(Thread):
    
    def __init__(self, user):
        Thread.__init__(self)
        self.f = None
        self.user = ""
        self.profile = "http://www.last.fm/user/"
        self.user = user
        self.artist = None
        self.title = None
        self.maintag_s = "<tr class=\"nowListening\">"
        self.maintag_e = "</tr>"
        self.pattern = re.compile("&(\w+?);")
        return
    
    def die(self, string):
        print string
        if self.f is not None:
            self.f.close()
        return
    
    def run(self):
        try:
            self.f = urllib.urlopen(self.profile + self.user)
        except:
            return self.die("Could not open %s" % (self.profile + self.user))
        if self.f is not None:
            try:
                raw = self.f.read(30000)
                start = raw.find(self.maintag_s)
                if start <= -1:
                    return self.die("Could not retrieve infos")
                end = raw.find(self.maintag_e, start + 1)
                if end <= -1:
                    return self.die("Could not find closing main-tag")
                raw_song = raw[start:end]
                if self.split_song(raw_song) == -1:
                    return self.die("Aborting.")
            finally:
                self.f.close()
        self.print_infos()
        return

    def get_artist(self, raw_song):
        artist = None
        ARTIST_BTAG = "<a href=\"/music/"
        ARTIST_ETAG = "</a>"
        ARTIST_LTAG = "\">"
        
        artist_s = raw_song.find(ARTIST_BTAG)
        if artist_s == -1:
            return self.die("Could not find artist-begining tag")
        artist_e = raw_song.find(ARTIST_ETAG, artist_s + 1)
        if artist_e == -1:
            return self.die("Could not find artist-ending tag")
        artist = raw_song[artist_s:artist_e]
        artist_s = artist.find(ARTIST_LTAG)
        if artist_s == -1:
            return self.die("Could not find artist-linking tag")
        artist = artist[artist_s + len(ARTIST_LTAG) :]
        return artist

    def get_title(self, raw_song):
        title = None
        TITLE_BTAG = "<a href=\"/music/"
        TITLE_ETAG = "</a>"
        TITLE_LTAG = "\">"

        title_s = raw_song.find(TITLE_BTAG)
        if title_s == -1:
            return self.die("Could not find title-begining tag")
        title_e = raw_song.find(TITLE_ETAG, title_s + 1)
        if title_e == -1:
            return self.die("Could not find title-ending tag")
        title = raw_song[title_s:title_e]
        title_s = title.find(TITLE_LTAG)
        if title_s == -1:
            return self.die("Could not find title-linking tag")
        title = title[title_s + len(TITLE_LTAG) :]
        if title == self.artist:
            return self.get_title(raw_song[title_s + len(TITLE_BTAG):])
        return title

    def split_song(self, raw_song):
        self.artist = self.get_artist(raw_song)
        self.title = self.get_title(raw_song)
        if self.artist == None or self.artist == None:
            return -1
        return 0

    def descape_entity(self, m, defs=htmlentitydefs.entitydefs):
        try:
            return defs[m.group(1)]
        except KeyError:
            return m.group(0)
        
    def descape(self, string):
        return self.pattern.sub(self.descape_entity, string)
   
    def print_infos(self):
        import xchat
        xchat.command("me is now listening to: %s %s %s" % (self.descape(self.artist), u"\u2013".encode("utf-8"), self.descape(self.title)))
        return    
    
def current_fm(word, word_eol, userdata):
    lfm = get_current_song("tusbar")
    print "Please wait... Processing."
    lfm.start()
    return xchat.EAT_NONE

def previous_fm(word, word_eol, userdata):
    lfm = get_previous_song("tusbar")
    print "Please wait... Processing."
    lfm.start()
    return xchat.EAT_NONE

print "%s %s loaded successfully" % (__module_name__, __module_version__)
xchat.hook_command("LFM", current_fm, help="Usage : /LFM\nShows the currently playing song on last.fm")
xchat.hook_command("PFM", previous_fm, help="Usage : /PFM\nShows the last played song on last.fm")
